% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictBranchpoints.R
\name{getCanonical3SS}
\alias{getCanonical3SS}
\title{Get locations of the first five AG 3' splice site motifs}
\usage{
getCanonical3SS(ag)
}
\arguments{
\item{ag}{Vector of distances to the AG splice site motif.}
}
\value{
Locations of the first five AG dinucleotides
}
\description{
Takes a variable length vector of distances to the AG motif,
sorts and returns the first five.
If there are less than five elements in the vector, returns the sorted vector
and fills the remainder of the values with 300.
}
\author{
Beth Signal
}
\keyword{internal}
